# -*- coding: utf-8 -*-
from django import forms
from django.conf import settings
from unfold.forms import AuthenticationForm
from django.utils.translation import gettext_lazy as _


class LoginForm(AuthenticationForm):
    username = forms.EmailField(
        label=_("Email"),
        max_length=254,
        widget=forms.EmailInput(attrs={"autocomplete": "email"}),
    )
    password = forms.CharField(widget=forms.PasswordInput(render_value=True))

    def __init__(self, request=None, *args, **kwargs):
        super().__init__(request, *args, **kwargs)

        if hasattr(settings, "DEFAULT_LOGIN_USERNAME") and hasattr(settings, "DEFAULT_LOGIN_PASSWORD"):
            self.fields["username"].initial = settings.LOGIN_USERNAME
            self.fields["password"].initial = settings.LOGIN_PASSWORD



import importlib
import logging
import pkgutil

from django.urls import reverse_lazy
from django.templatetags.static import static
from django.utils.translation import gettext_lazy as _

logger = logging.getLogger(__name__)


def extend_superapp_settings(main_settings, package):
    for importer, modname, ispkg in pkgutil.iter_modules(package.__path__):
        submodule_name = f"{package.__name__}.{modname}.settings"

        try:
            settings_module = importlib.import_module(submodule_name)
        except ModuleNotFoundError as e:
            logger.warning(e)
            continue

        if hasattr(settings_module, "extend_superapp_settings"):
            settings_module.extend_superapp_settings(main_settings)


def extend_with_superapp_settings(main_settings, superapp_apps):
    main_settings.update({
        'LOGIN_URL': "admin:login",
        'LOGIN_REDIRECT_URL': reverse_lazy("admin:index"),
    })
    main_settings['INSTALLED_APPS'] = [
                                          'admin_confirm',
                                          'unfold',
                                          "unfold.contrib.filters",
                                          "unfold.contrib.import_export",
                                          "unfold.contrib.guardian",
                                          "unfold.contrib.simple_history",
                                          "unfold.contrib.forms",
                                      ] + main_settings['INSTALLED_APPS']

    main_settings['UNFOLD'] = {
        "SITE_HEADER": _("SuperApp Demo"),
        "SITE_TITLE": _("SuperApp Demo"),
        "SITE_SYMBOL": "settings",
        "SHOW_HISTORY": False,
        "STYLES": [],
        "SCRIPTS": [
            lambda request: static("js/apex.min.js"),
            lambda request: static("js/flowbite.min.js"),
            lambda request: static("js/jquery-3.7.1.min.js"),
            lambda request: static("js/modals.js"),
        ],
    }

    extend_superapp_settings(main_settings, superapp_apps)



import copy
from typing import List, Optional, Dict, Any

from django.contrib.admin import helpers
from django.contrib.admin.utils import lookup_field
from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.db.models import ForeignKey
from django.forms import ModelChoiceField
from django.http import HttpRequest
from django.urls import reverse, URLPattern, path
from django.utils.text import wrap
from django_svelte_jsoneditor.widgets import SvelteJSONEditorWidget
from import_export.admin import ImportExportModelAdmin
from unfold.admin import ModelAdmin, UnfoldAdminReadonlyField
from unfold.contrib.forms.widgets import WysiwygWidget
from unfold.dataclasses import UnfoldAction
from unfold.widgets import UnfoldBooleanSwitchWidget, UnfoldAdminFileFieldWidget
from django.db import models

from admin_confirm import AdminConfirmMixin

from django_superapp.db_fields import ChainedForeignKey
from django_superapp.widgets import ChainedAdminSelect


class SuperAppAdminReadonlyField(UnfoldAdminReadonlyField):

    def is_custom_html_field(self) -> bool:
        field, obj, model_admin = (
            self.field["field"],
            self.form.instance,
            self.model_admin,
        )

        try:
            f, attr, value = lookup_field(field, obj, model_admin)
        except (AttributeError, ValueError, ObjectDoesNotExist):
            return False

        return hasattr(attr, 'custom_html_field') and attr.custom_html_field == True


helpers.AdminReadonlyField = SuperAppAdminReadonlyField

class ChainedForeignKeyAdmin:
    def formfield_for_foreignkey(
            self, db_field: ForeignKey, request: HttpRequest, **kwargs
    ) -> Optional[ModelChoiceField]:
        db = kwargs.get("using")

        if isinstance(db_field, ChainedForeignKey):
            kwargs["widget"] = ChainedAdminSelect(
                db_field, self.admin_site, using=db
            )
            pass
        return super().formfield_for_foreignkey(db_field, request, **kwargs)


class SuperAppModelAdmin(AdminConfirmMixin, ModelAdmin, ImportExportModelAdmin):
    actions_hidden = ()
    formfield_overrides = {
        models.TextField: {
            "widget": WysiwygWidget,
        },
        models.JSONField: {
            "widget": SvelteJSONEditorWidget,
        },
        models.FileField: {
            "widget": UnfoldAdminFileFieldWidget,
        },
    }

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = super().get_readonly_fields(request, obj)
        custom_html_fields = [
            attr for attr in dir(self)
            if hasattr(getattr(self, attr), 'custom_html_field') and getattr(self, attr).custom_html_field == True
        ]

        return tuple(readonly_fields) + tuple(custom_html_fields)

    def get_actions_hidden(self, request: HttpRequest) -> List[UnfoldAction]:
        return self._filter_unfold_actions_by_permissions(
            request, self._get_base_actions_hidden()
        )

    def _get_base_actions_hidden(self) -> List[UnfoldAction]:
        """
        Returns all available detail actions, prior to any filtering
        """
        return [self.get_unfold_action(action) for action in self.actions_hidden or []]

    def changeform_view(
            self,
            request: HttpRequest,
            object_id: Optional[str] = None,
            form_url: str = "",
            extra_context: Optional[Dict[str, bool]] = None,
    ) -> Any:
        if extra_context is None:
            extra_context = {}

        new_formfield_overrides = copy.deepcopy(self.formfield_overrides)
        new_formfield_overrides.update(
            {models.BooleanField: {"widget": UnfoldBooleanSwitchWidget}}
        )

        self.formfield_overrides = new_formfield_overrides

        actions = []
        if object_id:
            for action in self.get_actions_detail(request):
                actions.append(
                    {
                        "title": action.description,
                        "attrs": action.method.attrs,
                        "path": reverse(
                            f"admin:{action.action_name}", args=(object_id,)
                        ),
                    }
                )
            for action in self.get_actions_hidden(request):
                actions.append(
                    {
                        "title": action.description,
                        "attrs": action.method.attrs,
                        "path": reverse(
                            f"admin:{action.action_name}", args=(object_id,)
                        ),
                    }
                )

        extra_context.update(
            {
                "actions_submit_line": self.get_actions_submit_line(request),
                "actions_detail": actions,
            }
        )

        return super().changeform_view(request, object_id, form_url, extra_context)

    def get_urls(self) -> List[URLPattern]:
        urls = super().get_urls()

        action_hidden_urls = [
            path(
                f"<path:object_id>/{action.path}/",
                wrap(action.method),
                name=action.action_name,
            )
            for action in self._get_base_actions_hidden()
        ]
        return urls + action_hidden_urls


class BaseModel(models.Model):
    def clean(self):
        super().clean()
        for field in self._meta.fields:
            if isinstance(field, ChainedForeignKey):
                computed_filters = {}
                if field.dynamic_filters:
                    for key, value in field.filters.items():
                        computed_filters[key] = getattr(self, value).pk or None
                else:
                    computed_filters = field.filters
                if not field.remote_field.model.objects.filter(**{
                    **computed_filters,
                    'pk': getattr(self, field.name).pk or None
                }).exists():
                    raise ValidationError(
                        {
                            field.name: ValidationError(
                                f"Invalid value for {field.name}",
                                code='invalid'
                            )
                        }
                    )

    class Meta:
        abstract = True



from django.core.management.base import BaseCommand
from django.apps import apps


class Command(BaseCommand):
    help = 'Truncate all models of a specified app'

    def add_arguments(self, parser):
        parser.add_argument('app_name', type=str, help='Name of the Django app')

    def handle(self, *args, **kwargs):
        app_name = kwargs['app_name']
        app_models = apps.get_app_config(app_name).get_models()

        for model in app_models:
            model.objects.all().delete()

        self.stdout.write(self.style.SUCCESS('All models truncated successfully.'))



import importlib
import logging
import pkgutil

from django.urls import path

logger = logging.getLogger(__name__)

def extend_superapp_urlpatterns(main_urlpatterns, package):
    for importer, modname, ispkg in pkgutil.iter_modules(package.__path__):
        submodule_name = f"{package.__name__}.{modname}.urls"

        try:
            urls_module = importlib.import_module(submodule_name)
        except ModuleNotFoundError as e:
            logger.warning(e)
            continue

        if hasattr(urls_module, "extend_superapp_urlpatterns"):
            urls_module.extend_superapp_urlpatterns(main_urlpatterns)


def extend_superapp_admin_urlpatterns(main_admin_urlpatterns, package):
    for importer, modname, ispkg in pkgutil.iter_modules(package.__path__):
        submodule_name = f"{package.__name__}.{modname}.urls"

        try:
            urls_module = importlib.import_module(submodule_name)
        except ModuleNotFoundError as e:
            logger.warning(e)
            continue

        if hasattr(urls_module, "extend_superapp_admin_urlpatterns"):
            urls_module.extend_superapp_admin_urlpatterns(main_admin_urlpatterns)


main_admin_urlpatterns = []


def extend_with_superapp_urlpatterns(main_urlpatterns, superapp_apps):
    from django_superapp.sites import superapp_admin_site
    main_urlpatterns += [
        path("portal/", superapp_admin_site.urls),
    ]
    extend_superapp_urlpatterns(main_urlpatterns, superapp_apps)
    extend_superapp_admin_urlpatterns(main_admin_urlpatterns, superapp_apps)



import json

from django.contrib.admin.views.autocomplete import AutocompleteJsonView


class SmartSelectAutocompleteJsonView(AutocompleteJsonView):
    def get_queryset(self):
        """Return queryset based on ModelAdmin.get_search_results()."""
        qs = self.model_admin.get_queryset(self.request)
        qs = qs.complex_filter(self.source_field.get_limit_choices_to())
        qs, search_use_distinct = self.model_admin.get_search_results(
            self.request, qs, self.term
        )
        filters = json.loads(self.request.GET.get('filters', '{}'))

        qs = qs.filter(**filters)

        if search_use_distinct:
            qs = qs.distinct()
        return qs



from django.db.models import ForeignKey


class ChainedForeignKey(ForeignKey):
    def __init__(self, *args, **kwargs):
        self.filters = kwargs.pop('filters', {})
        self.dynamic_filters = kwargs.pop('dynamic_filters', True)
        self.automatically_select_unique_choice = kwargs.pop('automatically_select_unique_choice', False)
        super().__init__(*args, **kwargs)
